/***************************************************************************
 *   Copyright (C) 2008 by Gernot Pansy                                    *
 *   gernot@pansy.at                                                       *
 *                                                                         *
 *   Note:                                                                 *
 *    This code started off from a mix of StockWidget by Gerhard Gappmeier *
 *    and Stockoid by Jaagup Rep�n.                                        *
 *    So parts of this are also:                                           *
 *    Copyright (c) 2008 Gerhard Gappmeier & Jaagup Rep�n                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "stocks.h"
#include "ui_stocksConfig.h"
#include <QPainter>
#include <QFontMetrics>
#include <QStringList>
#include <KConfigDialog>
#include <KConfig>

#include <plasma/svg.h>
#include <plasma/theme.h>

#define STOCK_MIN_WIDTH 100
#define STOCK_SPACING 1

Stocks::Stocks(QObject *parent, const QVariantList &args) : Plasma::Applet(parent, args) {
    m_polling_interval = 300*1000,
    setBackgroundHints(DefaultBackground);
    setAspectRatioMode(Plasma::IgnoreAspectRatio);
    setHasConfigurationInterface(true);
    setMinimumSize(STOCK_MIN_WIDTH, 24);
}


Stocks::~Stocks() {
    if (hasFailedToLaunch()) {
        // Do some cleanup here
    } else {
        // Save settings
        KConfigGroup configGroup = config();
        configGroup.writeEntry("Stocks", stockList);
    }
}

void Stocks::init() {
  m_pStocksEngine = dataEngine("stocks");
  if (m_pStocksEngine) {
      
    kDebug() << "Loaded stocks dataEngine.";

    connect(m_pStocksEngine, SIGNAL(sourceAdded(const QString&)), this, SLOT(sltNewSource(const QString&)));
    connect(m_pStocksEngine, SIGNAL(sourceRemoved(const QString&)), this, SLOT(sltSourceRemoved(const QString&)));
    
    KConfigGroup configGroup = config();
    if (!(configGroup.readEntry("polling_interval", int()) < 1)) {
        m_polling_interval = configGroup.readEntry("polling_interval", int());
    }
    stockList = configGroup.readEntry("Stocks", QStringList());
    addStocks(stockList);
      
  } else {
      kDebug() << "Could not load stocks dataEngine.";
  }
}

void Stocks::paintInterface(QPainter *p, const QStyleOptionGraphicsItem *option, const QRect &contentsRect) {
    p->setRenderHint(QPainter::SmoothPixmapTransform);
    p->setRenderHint(QPainter::Antialiasing);

    QFont fontNormal = p->font();
    QFontMetrics fmNormal(fontNormal);
    int stockHeight = fmNormal.height()*2 + STOCK_SPACING;
    
    QSizeF appletSize = geometry().size();

    int minWidth = (int)appletSize.width()-contentsRect.width()+STOCK_MIN_WIDTH;
    int minHeight = (int)appletSize.height()-contentsRect.height()+stockHeight*m_hashStocks.size()-STOCK_SPACING;

    setMinimumSize(QSize(minWidth, minHeight));

    kDebug() << "Number of Stocks to draw: " << m_hashStocks.size();
    
    int i = 0;
    foreach (QString source, stockList) {
      if (m_hashStocks.contains(source)) {
        Stock* stock = m_hashStocks.value(source);
        stock->index = i;
        QRectF rowRect(contentsRect.left(), contentsRect.top()+i*stockHeight, contentsRect.width(), stockHeight);
        p->setFont(fontNormal);
        drawStock(p, stock, rowRect);
        i++;
      }
    }

    if ((int)appletSize.height() < minHeight) {
      resize((int)appletSize.width(), minHeight);
    }
}

/** This method draws a single stock line.
 * @param p Pointer to QPainter object.
 * @param pStock Pointer to stock information that should be drawn.
 */
void Stocks::drawStock(QPainter *p, Stock *pStock, const QRectF &stockRect) {
    QString sLabel;
    QString sValue = QString::number(pStock->fltValue, 'f', 2);;
    int valueWidth;
    QFont fontSmall;
    QFont fontNormal;
    QFontMetrics fmSmall(fontSmall);
    QFontMetrics fmNormal(fontNormal);
    
    fontNormal = p->font();
    fontSmall = p->font();
    fontNormal.setPointSize(fontNormal.pointSize()+2);
    fontNormal.setBold(TRUE);
    fontSmall.setPointSize(fontSmall.pointSize()-1);

    QRectF rowRect(stockRect.left()+2, stockRect.top(), stockRect.width()-4, fmNormal.height());
    
    // draw stock ID
    p->setFont(fontNormal);
    p->setPen(Qt::white);
    p->drawText(rowRect, Qt::AlignLeft | Qt::AlignTop, pStock->sId);

    // draw font value
    p->setFont(fontNormal);
    p->setPen(Qt::white);
    p->drawText(rowRect, Qt::AlignRight | Qt::AlignTop, sValue);

    rowRect.setTop(stockRect.top() + fmNormal.height());
    rowRect.setHeight(fmSmall.height());
    
    p->setFont(fontSmall);
    if (pStock->fltChange < 0.0) {
        p->setPen(Qt::red);
        sValue = QString("%1").arg(pStock->fltChange, 0, 'f', 2) + 
          QString(" (%1%)").arg(pStock->fltPercentChange, 0, 'f', 2);
    } else if (pStock->fltChange > 0.0) {
        p->setPen(Qt::green);
        sValue = QString("+%1").arg(pStock->fltChange, 0, 'f', 2) +
          QString(" (+%1%)").arg(pStock->fltPercentChange, 0, 'f', 2);
    } else {
      sValue = ""; /*QString("%1").arg(pStock->fltChange, 0, 'f', 2) +
          QString(" (%1%)").arg(pStock->fltPercentChange, 0, 'f', 2);*/
    }
    p->drawText(rowRect, Qt::AlignRight | Qt::AlignTop, sValue);

    if (stockRect.width() >= 120) {
      // calculate value width
      valueWidth = fmSmall.width(sValue);
      if (pStock->fltValue == 0.0) {
          sLabel = tr("Waiting for data...");
      } else {
          sLabel = pStock->sName;
      }
      // shorten label if necessary
      if (fmSmall.width(sLabel) + valueWidth > rowRect.width()) {
          sLabel = sLabel.left(sLabel.length() - 3) + "...";
          while (sLabel.length() > 6 && (fmSmall.width(sLabel) + valueWidth) > rowRect.width()) {
              sLabel = sLabel.left(sLabel.length() - 4) + "...";
          }
      }

      // draw stock name
      p->setFont(fontSmall);
      p->setPen(Qt::lightGray);
      p->drawText(rowRect, Qt::AlignLeft | Qt::AlignTop, sLabel);
    }
}

void Stocks::createConfigurationInterface(KConfigDialog *parent) {
  QWidget *widget = new QWidget;
  m_ui.setupUi(widget);
  parent->setButtons(KDialog::Ok | KDialog::Cancel);
  connect(parent, SIGNAL(okClicked()), this, SLOT(configAccepted()));
  parent->addPage(widget, parent->windowTitle(), icon());
  m_ui.Stocks->setItems(stockList);
}

void Stocks::configAccepted() {
  stockList = m_ui.Stocks->items();
  removeStocks(stockList);
  addStocks(stockList);
  emit configNeedsSaving();
}

void Stocks::addStocks(const QStringList &stockList) {
  QString source;
  foreach (source, stockList) {
    if (!m_hashStocks.contains(source)) {
      m_pStocksEngine->connectSource(source, this, m_polling_interval);
    }
  }
}

void Stocks::removeStocks(const QStringList &stockList) {
  QString source;
  foreach (source, m_hashStocks.keys()) {
    if (!stockList.contains(source)) {
      m_pStocksEngine->disconnectSource(source, this);
    }
  }
}

void Stocks::sltNewSource(const QString &source) {
    
  kDebug() << "source =" << source;
  
  Stock* newStock = new Stock();
  newStock->sId = source;
  newStock->sName = "";
  newStock->fltValue =  0.0;
  newStock->fltChange = 0.0;
  newStock->fltPercentChange = 0.0;
  
  m_hashStocks.insert(source, newStock);
 }

void Stocks::sltSourceRemoved(const QString &source) {

  kDebug() << "source =" << source;
  
  m_hashStocks.remove(source);
}

void Stocks::dataUpdated(const QString &source, Plasma::DataEngine::Data data) {
    
  kDebug() << "source =" << source;
  
  if (m_hashStocks.contains(source)) {
    kDebug() << "Updating Value =" << data["Value"].toDouble();
    Stock* stock = m_hashStocks.value(source);
    stock->sName = data["Name"].toString();
    stock->fltValue = data["Value"].toDouble();
    stock->fltChange = data["Change"].toDouble();
    stock->fltPercentChange = data["PercentChange"].toDouble();
  }
  update();
}


#include "stocks.moc"
